<?php
/**
 * REST API Validate_Login controller
 *
 * Handles requests to the /validate-login endpoint.
 *
 * @author   WooThemes
 * @category API
 * @package  WooCommerce/API
 * @since    2.6.0
 */

if ( ! defined( 'ABSPATH' ) ) {

	exit;
}
/**
 * REST API Validate_Login controller class.
 *
 * @package WooCommerce/API
 * @extends WC_REST_Controller
 */
class WC_REST_Validate_Login_Controller extends WC_REST_Controller {

	/**
	 * Endpoint namespace.
	 *
	 * @var string
	 */
	protected $namespace = 'wc/v2';

	/**
	 * Route base.
	 *
	 * @var string
	 */
	protected $rest_base = 'validate-login';

	/**
	 * Post type.
	 *
	 * @var string
	 */
	protected $post_type = '';

	/**
	 * If object is hierarchical.
	 *
	 * @var bool
	 */
	protected $hierarchical = true;

	/**
	 * Stores the request.
	 * @var array
	 */
	protected $request = array();

	/**
	 * Register the routes for orders.
	 */
	public function register_routes() {

		register_rest_route( $this->namespace, '/' . $this->rest_base, array(
			array(
				'methods'             => WP_REST_Server::READABLE,
				'callback'            => array( $this, 'validate_user' ),
				//'permission_callback' => array( $this, 'get_items_permissions_check' ),
				//'args'                => $this->get_collection_params(),
			),
			array(
				'methods'             => WP_REST_Server::CREATABLE,
				'callback'            => array( $this, 'validate_user' ),
				//'permission_callback' => array( $this, 'create_item_permissions_check' ),
			),
			//'schema' => array( $this, 'get_public_item_schema' ),
		) );
	}


	/**
	 * Get a collection of posts.
	 *
	 * @param WP_REST_Request $request Full details about the request.
	 * @return WP_Error|WP_REST_Response
	 */
	public function validate_user( $request ) {

		$user_email        = $request['app_owner_email'];
		//$user_username       = $request['app_owner_username'];
		$user_password        = $request['app_owner_password'];

		// for validations get details of user by email address
		$user_data = get_user_by( 'email', $user_email );

		// check if email is valid
		if ( empty( $user_email ) || empty( $user_data->ID ) ) {
			return new WP_Error( 'woocommerce_rest_invalid_user', __( 'Invalid user email id.', 'woocommerce' ), array( 'status' => 404 ) );
		}

		// check passed password is valid 
		if ( empty($user_password) || !wp_check_password( $user_password, $user_data->user_pass, $user_data->ID) ) {
			return new WP_Error( 'woocommerce_rest_invalid_user_password', __( 'Invalid user password.', 'woocommerce' ), array( 'status' => 404 ) );
		}

		// auto login functionality 
		$auth_userinfo = wp_authenticate($user_email, $user_password);
		// first destroy all sessions then set
		$sessions = WP_Session_Tokens::get_instance( $auth_userinfo->ID );
		$sessions->destroy_all();
		// set new session
		wp_set_current_user ( $auth_userinfo->ID );
    	wp_set_auth_cookie  ( $auth_userinfo->ID );

		// Manage user sessions
		$sessions_meta = get_user_meta( $user_data->ID, 'session_tokens', true );
		
		if ( !empty($sessions_meta) ) {

			if ( count( $sessions_meta ) === 1 ) {

				$get_last_session_key = key($sessions_meta);
				// assign key as session token
				$session_token = $get_last_session_key;

			} else if ( count( $sessions_meta ) > 1 ) {

				// user must be logged in
				return new WP_Error( 'woocommerce_rest_multiple_user_session', __( 'Did you lose your phone or leave your account logged in at a public computer? You can log out everywhere else, and stay logged in one place. Go to Wordpress Dashboard > Users > Your Profile > Account Management > Sessions. And then revalidate your login from app.', 'woocommerce' ), array( 'status' => 404 ) );
			}

		} else {
			// user must be logged in
			return new WP_Error( 'woocommerce_rest_invalid_user_session', __( 'User must be logged in.', 'woocommerce' ), array( 'status' => 404 ) );
		}
		// Manage user sessions

		$user_data = $user_data->to_array();
		$user_data['session_token'] = $session_token;

		$user = $this->prepare_item_for_response( $user_data, $request );
		$response = rest_ensure_response( $user );

		return $response;
	}


	/**
	 * Prepare a single customer output for response.
	 *
	 * @param  WP_User          $user_data User object.
	 * @param  WP_REST_Request  $request Request object.
	 * @return WP_REST_Response $response Response data.
	 */
	public function prepare_item_for_response( $user_data, $request ) {
		
		$data = array(
			'id'            => $user_data['ID'],
			'date_created'  => $user_data['user_registered'],
			'date_modified' => null,
			'email'         => $user_data['user_email'],
			'display_name'    => $user_data['display_name'],
			'username'      => $user_data['user_login'],
			'avatar_url'    => get_avatar_url( $user_data['ID'] ),
			'session_token'    => $user_data['session_token'],
			'app_owner_password'    => $request['app_owner_password'],
		); 

		$context  = ! empty( $request['context'] ) ? $request['context'] : 'view';
		$data     = $this->add_additional_fields_to_object( $data, $request );
		$data     = $this->filter_response_by_context( $data, $context );
		$response = rest_ensure_response( $data );
		$response->add_links( $this->prepare_links( $user_data ) );

		/**
		 * Filter customer data returned from the REST API.
		 *
		 * @param WP_REST_Response $response   The response object.
		 * @param WP_User          $user_data  User object used to create response.
		 * @param WP_REST_Request  $request    Request object.
		 */
		return apply_filters( 'woocommerce_rest_prepare_valid_user', $response, $user_data, $request );
	}

	/**
	 * Prepare links for the request.
	 *
	 * @param WP_User $customer Customer object.
	 * @return array Links for the given customer.
	 */
	protected function prepare_links( $user ) {
		$links = array(
			'self' => array(
				'href' => rest_url( sprintf( '/%s/%s/%d', $this->namespace, $this->rest_base, $user->ID ) ),
			),
			'collection' => array(
				'href' => rest_url( sprintf( '/%s/%s', $this->namespace, $this->rest_base ) ),
			),
		);

		return $links;
	}

}